@@ -10,12 +10,13 @@ if ($@) {
     @ISA = qw(CGI::XA);
 }
 
-$VERSION = (qw$Revision: 1.20 $)[1];
+$VERSION = sprintf("%d.%02d", q$Revision: 1.16 $ =~ /(\d+)\.(\d+)/);
 
 sub new {
     my($class) = shift;
+    #tie *STDOUT => 'Apache::TieHandle';
     my($r) = Apache->request;
-    %ENV = $r->cgi_env unless $ENV{GATEWAY_INTERFACE}; #PerlSetupEnv On 
+    %ENV = $r->cgi_env;
     my $self = $class->SUPER::new(@_);
     $self->{'.req'} = $r;
     $self;
@@ -25,7 +26,7 @@ sub header {
     my $self = shift;
     my $r = $self->{'.req'};
     $r->basic_http_header;
-    return $self->CGI::XA::header(@_);
+    return $self->CGI::XA::header(@hdrs);
 }		     
 
 sub print {
@@ -36,7 +37,23 @@ sub print {
 sub read_from_client {
     my($self, $fh, $buff, $len, $offset) = @_;
     my $r = $self->{'.req'} || Apache->request;
-    return $r->read($$buff, $len, $offset);
+    my($own_buffer,$ret);
+    # A.K.: unfortunately I couldn't find a way without our own buffer
+    $ret = $r->read_client_block($own_buffer, $len);
+#    my $own_len = length($own_buffer);
+    if ($offset) {
+	substr($$buff,$offset) = substr($own_buffer,0,$ret);
+    } else {
+	$$buff = substr($own_buffer,0,$ret);
+    }
+#    my $fulllen = length($$buff);
+#    my $caller = "";#Carp::longmess();
+#    warn qq{ret [$ret] len [$len] offset [$offset] own_len [$own_len] fulllen [$fulllen] caller [$caller]\n};
+#    my $obs = substr($own_buffer,0,15);
+#    my $obe = substr($own_buffer,-15);
+#    for ($obs,$obe){ s/\000/\\000/g }
+#    warn qq{own_buffer [$obs]...[$obe] his buff [$$buff]\n};
+    $ret;
 }
 
 sub new_MultipartBuffer {
@@ -48,11 +65,11 @@ sub new_MultipartBuffer {
 
 sub exit {
     my($self, $s) = @_;
+    $self->{'.req'}->connection->close;
     $self->{'.req'}->exit($s);
 }
 
 package Apache::MultipartBuffer;
-use vars qw(@ISA);
 @ISA = qw(CGI::XA::MultipartBuffer);
 
 sub wouldBlock { undef }
